# Production Dat
#----------------------------
# Global Variable Definitions
#----------------------------
#
debug=3
log_stderror=no
log_facility=LOG_LOCAL7
fork=1

check_via=0
dns=1
rev_dns=0
children=4
fifo="/tmp/openser_fifo"
user="openser"
group="openser"
sock_mode=0666
fifo_db_url="mysql://openser:openser@localhost/openser"

#----------------------------------------------------------------
# The port parameter specifies which TCP/UDP port OPENSER should 
# run. The default is 5060 according to RFC 3261.
#----------------------------------------------------------------
port=5060
alias="upcnet.es"
listen=udp:127.0.0.1
listen="147.83.197.39"

#----------------------------------------------------------------
# Module Loading
#----------------------------------------------------------------

mpath="/usr/lib/openser/modules"

loadmodule "mysql.so"
loadmodule "sl.so"
loadmodule "tm.so"
loadmodule "rr.so"
loadmodule "maxfwd.so"
loadmodule "usrloc.so"
loadmodule "registrar.so"
loadmodule "textops.so"
loadmodule "domain.so"
loadmodule "group.so"
loadmodule "auth.so"
loadmodule "auth_db.so"
loadmodule "permissions.so"
loadmodule "msilo.so"
loadmodule "uri.so"
loadmodule "uri_db.so"
loadmodule "xlog.so"
loadmodule "mediaproxy.so"
loadmodule "nathelper.so"

#-----------------------------------------------------------------
# Authentication Parameters
#-----------------------------------------------------------------
modparam("auth_db", "db_url", "mysql://openser:openser@localhost/openser")
modparam("auth_db", "calculate_ha1", 1)
modparam("auth_db", "password_column", "password")
modparam("auth_db", "use_domain", 1)

#-----------------------------------------------------------------
# User Location Parameters
#-----------------------------------------------------------------
modparam("usrloc", "db_url", "mysql://openser:openser@localhost/openser")
modparam("usrloc", "db_mode", 2) 
modparam("usrloc", "use_domain", 1)

#-----------------------------------------------------------------
# Domain Parameters
#-----------------------------------------------------------------
modparam("domain", "db_url", "mysql://openser:openser@localhost/openser")
modparam("domain", "db_mode", 1)

#-----------------------------------------------------------------
# Record Route Parameters
#-----------------------------------------------------------------
modparam("rr", "enable_full_lr", 1)

#-----------------------------------------------------------------
# Permission Parameters
#-----------------------------------------------------------------
modparam("permissions", "db_url", "mysql://openser:openser@localhost/openser")
modparam("permissions", "trusted_table", "trusted")

#-----------------------------------------------------------------
# Permission Parameters
#-----------------------------------------------------------------
modparam("registrar", "use_domain", 1)
modparam("registrar", "nat_flag", 6)

#-----------------------------------------------------------------
# URI_DB Parameters
#-----------------------------------------------------------------
modparam("uri_db", "db_url", "mysql://openser:openser@localhost/openser")

#-----------------------------------------------------------------
# GROUP Parameters
#-----------------------------------------------------------------
modparam("group", "db_url", "mysql://openser:openser@localhost/openser")
modparam("group", "table", "grp")
modparam("group", "use_domain", 1)

#-----------------------------------------------------------------
# MEDIAPROXY Parameters
#-----------------------------------------------------------------
modparam("nathelper", "rtpproxy_disable", 1)
modparam("nathelper", "natping_interval", 0)

#--------------------------------------------------------------
# MAIN ROUTE
#--------------------------------------------------------------
route {

  route(22);

  #------------------------------------------------------------
  # Sanity Check Section
  #------------------------------------------------------------
  if (!mf_process_maxfwd_header("10")) {
    sl_send_reply("483", "Too Many Hops");
    exit;
  };

  if (msg:len > max_len) {
    sl_send_reply("513", "Message Overflow");
    exit;
  };
  
  #------------------------------------------------------------
  # Record Route Section
  #------------------------------------------------------------
  if (is_method("INVITE") && client_nat_test("3")) 
  	add_rr_param(";nat=yes");

  if (method!="REGISTER")
	record_route();

  #------------------------------------------------------------
  # Loose Route Section
  #------------------------------------------------------------
  if (is_method("BYE") || is_method("CANCEL"))
	end_media_session();

  #------------------------------------------------------------
  # Loose Route Section
  #------------------------------------------------------------
  if (loose_route()) {
	if ( has_totag() && ( is_method("INVITE") || is_method("ACK") ) ) {
		if (client_nat_test("3") || search("^Route:.*;nat=yes")) {
			setflag(9);
			use_media_proxy();
		};
	};
	route(1);
        exit;
  };


  #------------------------------------------------------------
  # Message Processing Section
  #------------------------------------------------------------
  if (is_method("ACK")) {
	route(1);
	exit;
  };

  if (is_method("CANCEL")) {
	route(1);
	exit;
  };

  if (is_method("REGISTER")) {
	route(13);
	exit;
  };

  if (uri!=myself) {
	route(19);
	route(1);
	exit;
  };

  if (is_method("INVITE")) {
	route(6);
	exit;
  };

  lookup("aliases");
  if (uri!=myself) {
	route(19);
	route(1);
	exit;
  };

  if (!lookup("location")) {
    # User not found
    sl_send_reply("404", "User Not Found");
    exit;
  };

  route(1);
}

# -----------------------------------------------------------------
# Default Message Handler
# -----------------------------------------------------------------
route[1] {
	t_on_reply("1");

	if (!t_relay()) {
		if (is_method("INVITE") || is_method("ACK"))
			end_media_session();

		sl_reply_error();
	};
}

#--------------------------------------------------------------
# ROUTE REGISTER
#--------------------------------------------------------------
route[13] {

  #-------------------------------------------------------------
  # Let the UA know we are working on their request - they shouldn't 
  # send retransmissions
  #-------------------------------------------------------------
  sl_send_reply("100", "Trying");

  #--------------------------------------------------------------------
  # Check for NAT
  #--------------------------------------------------------------------
  if (!search("^Contact:[ ]*\*") && client_nat_test("7")) {
	if (isflagset(1))
        	xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu is a NAT client\n");

	setflag(9);
	fix_nated_register();
	force_rport();
  };

  #--------------------------------------------------------------------
  # Check for known domain(s)
  #--------------------------------------------------------------------
  if (!is_from_local()) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu Unknown domain\n");
    sl_send_reply("403", "Unknown domain");
    return(-1);
  };

  #--------------------------------------------------------------------
  # Check to see that the user has not been disabled
  #--------------------------------------------------------------------
  if (is_user_in("From", "disabled")) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu Your account has been disabled\n");
    sl_send_reply("403", "Your account has been disabled");
    exit;
  };
  if (is_user_in("To", "disabled")) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $tu Your account has been disabled\n");
    sl_send_reply("403", "Your account has been disabled");
    return(-1);
  };

  #--------------------------------------------------------------------
  # Prohibit attempts to grab someone else's To address using valid credentials
  #--------------------------------------------------------------------
  if (!www_authorize("", "subscriber")) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu Authorization for $fu ($si) requested\n");
    www_challenge("", "0");
    return(-1);
  } else {
    if (isflagset(1))
       xlog("L_NOTICE","SIP@UPC: $mi route[$rm][13] $fu Authorization OK\n");
  };

  #--------------------------------------------------------------------
  # check_to
  #--------------------------------------------------------------------
  if (!check_to()) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu -> $ru REPLY 401 - Unauthorized check_to() failed\n");
    sl_send_reply("401", "Unauthorized");
    return(-1);
  };

  consume_credentials();

  #--------------------------------------------------------------------
  # Authenticated request, update location table
  #--------------------------------------------------------------------
  if (!save("location")) {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu save(location) Failed\n");
    sl_reply_error();
    return(-1);
  } else {
    if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][13] $fu save(location)OK\n");

    # Sends stored messages to SIP UAC
    m_dump();
  };

  # return(TRUE);
}

# -----------------------------------------------------------------
# INVITE Message Handler
# -----------------------------------------------------------------
route[6] {

  #-------------------------------------------------------------
  # Let the UA know we are working on their request - they shouldn't 
  # send retransmissions
  #-------------------------------------------------------------
  sl_send_reply("100", "Trying");

  #-------------------------------------------------------------
  # If UA is not trusted, it must authenticate
  #-------------------------------------------------------------
  if (!allow_trusted()) {

    if (!proxy_authorize("", "subscriber")) {
      if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] $fu Authorization for $fu ($si) requested\n");
      proxy_challenge("", "0");
      return(-1);
    };

    if (!check_from()) {
      sl_send_reply("403", "Use From=ID");
      return(-1);
    }; 

    consume_credentials();
  };

  #--------------------------------------------------------------------
  # Check for NAT
  #--------------------------------------------------------------------
  if (client_nat_test("3")) {
	if (isflagset(1))
        	xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] $fu is a NAT client\n");

	setflag(4);
	force_rport();
	fix_nated_contact();
	force_rport();
  };

  #------------------------------------------------------------
  # Call from PBX Gateway
  #------------------------------------------------------------
  if (allow_trusted() && uri=~"^sip:[0-9]*@") {		
    rewritehost("upcnet.es");
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] PBX Gateway is calling $tu\n");
  };

  #--------------------------------------------------------------------
  # External Calls ti PSTN must begin with '0' digit
  #--------------------------------------------------------------------
  if (uri=~"^sip:0[0-9]+@.*") {		
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] $fu is calling to $tu via PSTN\n");
    route(19);
    route(20);
    exit;
  };
  
  #--------------------------------------------------------------------
  # Try to find any matching alias in our alias DB
  #--------------------------------------------------------------------
  lookup("aliases");

  #--------------------------------------------------------------------
  # If CALLEE is in other domain. Proxy it
  #--------------------------------------------------------------------
  if (uri!=myself) {
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] $fu is calling to $tu via proxy\n");

    route(19);
    route(1);
    exit;
  };

  if (!lookup("location")) {
    sl_send_reply("404", "User Not Found");
    return(-1);
  };

  if (isflagset(1))
    xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][6] $fu is calling to $tu via me\n");

  route(19);
  route(1);
}

# -----------------------------------------------------------------
# NAT Traversal
# -----------------------------------------------------------------
route[19] {
  if (isflagset(9) || isflagset(4)) 
	use_media_proxy();
}

# -----------------------------------------------------------------
# PSTN Handler
# -----------------------------------------------------------------
route[20] {

  if (uri=~"^sip:0[1-9][0-9]{4}@.*") {
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu making PBX call ($tu)\n");
    setflag(2);
    if (is_user_in("credentials", "nocalls")) {
      if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu hasn't CALL permission\n");
      sl_send_reply("403", "International permissions needed");
      return(-1);
    };
  # Is user allowed for NATIONAL calls? (destinations begin with two zeros)
  } else if (uri=~"^sip:0[1-9][0-9]+@.*") {
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu making NATIONAL call ($tu)\n");
    setflag(2);
    if (!is_user_in("credentials", "national")) {
      if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu  hasn't NATIONAL permission\n");
      sl_send_reply("403", "National permissions needed");
      return(-1);
    };
  # the same for INTERNATIONAL calls (three zeros)
  } else if (uri=~"sip:000[1-9][0-9]+@.*") {
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu making INTERNATIONAL call ($tu)\n");
    setflag(2);
    if (!is_user_in("credentials", "international")) {
      if (isflagset(1))
        xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu  hasn't INTERNATIONAL permission\n");
      sl_send_reply("403", "International permissions needed");
      return(-1);
     };
  # everything else (e.g., interplanetary calls) is denied
  } else {
    if (isflagset(1))
      xlog("L_NOTICE", "SIP@UPC: $mi route[$rm][20] $fu making INCORRECT call ($tu)\n");
    sl_send_reply("403", "Forbidden");
    return(-1);
  };

  rewritehost("pstnsip.upc.es:5060");
  route(1);
}

# -----------------------------------------------------------------
# ONREPLY ROUTES
# -----------------------------------------------------------------
onreply_route[1] {
  if ((isflagset(9) || isflagset(4)) && (status=~"(180)|(183)|2[0-9][0-9]")) {
    if (!search("^Content-Length:[ ]*0")) {
      use_media_proxy();
    };
  }; 

  if (client_nat_test("1")) {
    fix_nated_contact();
  };
}

#--------------------------------------------------------------
# ROUTE DEBUG FLAGS
#--------------------------------------------------------------
route[22] {
  resetflag(1);
  #------------------------------------------------------------
  # Methods that will have debugging turned on
  #------------------------------------------------------------
       if (is_method("ACK"))           { resetflag(1); }
  else if (is_method("BYE"))           { setflag(1); }
  else if (is_method("CANCEL"))        { setflag(1); }
  else if (is_method("INVITE"))        { setflag(1); }
  else if (is_method("OPTIONS"))       { setflag(1); }
  else if (is_method("PUBLISH"))       { setflag(1); }
  else if (is_method("REGISTER"))      { setflag(1); }
  else if (is_method("MESSAGE"))       { setflag(1); }
  else if (is_method("NOTIFY"))        { setflag(1); }
  else if (is_method("SUBSCRIBE"))     { setflag(1); }
  else if (is_method("INFO"))          { setflag(1); }
  else if (is_method("PRACK"))         { setflag(1); }
  else if (is_method("REFER"))         { setflag(1); }
  else {
     setflag(1);
  };
}
